// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "board.h"

extern GLfloat light_0_shadow[4][4];

// Class of the game board
// ----------------------------------------------------------------------------

// constructor, proceed with board initialization
Board::Board(int x, int y)
{
    set_color_array(seed_ambient_and_diff, 0.8, 0.5, 0.5);
    seed_shininess[0] = 40.0;
    set_color_array(seed_specular, 1.0, 0.8, 0.8, 1.0);

    set_color_array(shadow_color, 0.2, 0.2, 0.3);

    x_size = x;
    y_size = y;

    // create the array
    cells = new Cell**[x];
    for (int i = 0; i < x; i++)
    {
        cells[i] = new Cell*[y];
        // create the cells
        for (int j = 0; j < y; j++)
            cells[i][j] = new Cell(i, j);
    }
}

// reinitialize
void Board::init(void)
{
    // reinit the cells
    for (int i = 0; i < x_size; i++)
        for (int j = 0; j < y_size; j++)
            cells[i][j]->set_free();
}

// Destroyer, delete cells
Board::~Board(void)
{
    // destruction des cellules
    for (int i = 0; i < x_size; i++)
        for (int j = 0; j < y_size; j++)
            delete cells[i][j];

    // destruction du tableau
    delete[] cells;
}

// Create a random seed on the board
void Board::create_random_seed(void)
{
    do
    {
        x_pos = (int) ((x_size - 1) * (float) rand() / (float) RAND_MAX);
        y_pos = (int) ((y_size - 1) * (float) rand() / (float) RAND_MAX);
    }
    while (!this->cells[x_pos][y_pos]->is_free());
}

// get the position of the seed on the board
void Board::get_seed_pos(int &seed_x, int &seed_y)
{
    seed_x = x_pos;
    seed_y = y_pos;
}

// compare given position with the one of the seed
bool Board::is_on_seed_cell(int x, int y)
{
    return (x == x_pos && y == y_pos);
}

// display the world
void Board::proceed_display(void) {
    //put_borders ();

    // display cells
    for (int i = 0; i < this->x_size; i++)
        for (int j = 0; j < this->y_size; j++)
            cells[i][j]->proceed_display();

    // display seed
    glPushMatrix();

        glTranslatef(10.0 * this->x_pos, 10.0 * this->y_pos, 0.0);

        glTranslatef(5.0, 5.0, 0.0);

        // display the seed's shadow
        glDisable(GL_LIGHTING);
        glColor3fv(shadow_color);

        glPushMatrix();
            glTranslatef(0.0, 0.0, 0.1);
            glMultMatrixf((GLfloat *) light_0_shadow);
            glTranslatef(0.0, 0.0, 3.0);
            glutSolidSphere(3.0, 20.0, 10.0);
        glPopMatrix();

        glEnable(GL_LIGHTING);

        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, seed_ambient_and_diff);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, seed_shininess);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, seed_specular);

        //glTranslatef (5.0, 5.0, 3.0);
        glTranslatef(0.0, 0.0, 3.0);
        glutSolidSphere(3.0, 20.0, 20.0);

    glPopMatrix();
}

int Board::get_x_size()
{
    return this->x_size;
}

int Board::get_y_size()
{
    return this->y_size;
}

Cell* Board::get_cell (int x, int y)
{
    return this->cells[x][y];
}
