// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "camera.h"

extern Board* board;

// class of the camera
// ----------------------------------------------------------------------------

//constructor
Camera::Camera(void)
{
    set_color_array(position, 0.0 * board->get_x_size(), 0.0, 8.0 * board->get_x_size());
    set_color_array(direction, 0.0, 0.0, 0.0);
    set_color_array(up_vector, 0.0, 1.0, 0.0);

    spin = 0;
    do_rotation = true;

    if (speed_coeff_cam == 0.0)
        speed_coeff_cam = 1.0;
}

// specify the value of a camera move increment
void Camera::save_value_coeff(float coeff)
{
    speed_coeff_cam = coeff;
}

// reinit camera
void Camera::init(void)
{
    speed_coeff_cam = 1.0;
}

// start / stop camera rotation
void Camera::start_stop_rotation(void)
{
    do_rotation = !do_rotation;
}

// position the camera
void Camera::proceed_position(void)
{
    double angle;

    if (do_rotation)
    {
        spin += speed_coeff_cam / 3;

        if (spin > 360)
        {
            spin = 0;
        }
    }

    // angle in radian =
    angle = (my_pi * (double) spin) / 180.0;

    position[0] = (GLfloat) (cos(angle) * 6.0) * (GLfloat) board->get_x_size();// + x;
    position[1] = (GLfloat) (sin(angle) * 6.0) * (GLfloat) board->get_x_size();// + y;

    up_vector[0] = -1.0 * (GLfloat) (cos(angle));
    up_vector[1] = -1.0 * (GLfloat) (sin(angle));

    gluLookAt(position[0], position[1], position[2], direction[0],
            direction[1], direction[2], up_vector[0], up_vector[1],
            up_vector[2]);

}

// reshape the camera
void Camera::reshape(GLsizei width, GLsizei height)
{

    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum(-4.0 * board->get_x_size(), 4.0 * board->get_x_size(), -4.5 * board->get_y_size(),
               3.0 * board->get_y_size(), 4.0 * board->get_x_size(), 15.0 * board->get_x_size());

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

