// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "cell.h"
#include "board.h"


extern Board* board;

// A cell on the game board
// -----------------------------------------------------------------------------

// default constructot
Cell::Cell(void)
     :Display_Object()
{
}

// constructor specifying the cell position
Cell::Cell(int x, int y)
     :Display_Object(x, y)
{
    mat_shininess[0] = 20.0;
    set_color_array(mat_specular, 0.7, 1.0, 0.8, 1.0);
    set_color_array(mat_ambient_and_diff, 0.1, 0.3, 0.2);

    ground_shininess[0] = 20.0;
    set_color_array(ground_specular, 1.0, 1.0, 1.0, 1.0);
    set_color_array(ground_ambient_and_diff, 0.3, 0.6, 0.6);

    free = true;

    // create the display list
    display_list = glGenLists(1);
    glNewList(display_list, GL_COMPILE);
        this->proceed_display();
    glEndList();
}

// destroyer
Cell::~Cell(void)
{
}

// used to know whether the cell is free
bool Cell::is_free(void)
{
    return free;
}

// free the cell
void Cell::set_free(void)
{
    free = true;
}

// mark the cell as busy
void Cell::set_used(void)
{
    free = false;
}

std::vector<Cell*>* Cell::get_neighbours()
{
    std::vector<Cell*>* ret_list = new std::vector<Cell*>();

    if (this->x_pos > 0)
    {
        ret_list->push_back(board->get_cell(x_pos - 1, y_pos));
    }

    if (this->y_pos > 0)
    {
        ret_list->push_back(board->get_cell(x_pos, y_pos - 1));
    }

    if (this->x_pos < board_size - 1)
    {
        ret_list->push_back(board->get_cell(x_pos + 1, y_pos));
    }

    if (this->y_pos < board_size - 1)
    {
        ret_list->push_back(board->get_cell(x_pos, y_pos + 1));
    }

    return ret_list;
}

direction_tp Cell::get_cell_direction (Cell* cell)
{
    if (x_pos < cell->x_pos)
    {
        return right;
    }
    else if (x_pos > cell->x_pos)
    {
        return left;
    }
    else if (y_pos < cell->y_pos)
    {
        return up;
    }
    else
    {
        return down;
    }
}

// cell display func
void Cell::proceed_display(void)
{
    glPushMatrix();

        glTranslatef(10.0 * this->x_pos, 10.0 * this->y_pos, 0.0);

        // draw the parcel square
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ground_ambient_and_diff);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, ground_shininess);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, ground_specular);

        draw_square();

        // draw the little square
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ambient_and_diff);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);

        glTranslatef(0.0, 0.0, 0.1);
        draw_little_square();

    glPopMatrix();
}

