// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "common.h"

void set_color_array(GLfloat* tab, GLfloat red, GLfloat green, GLfloat blue)
{
    tab[0] = red;
    tab[1] = green;
    tab[2] = blue;
}

void set_color_array(GLfloat* tab, GLfloat red, GLfloat green, GLfloat blue,
        GLfloat alpha)
{
    tab[0] = red;
    tab[1] = green;
    tab[2] = blue;
    tab[3] = alpha;
}

void compute_normal_vector(GLfloat* point_1, GLfloat* point_2, GLfloat* point_3,
        GLfloat norm[3])
{
    GLfloat vect_1[] = { point_1[0] - point_2[0], point_1[1] - point_2[1],
            point_1[2] - point_2[2] };

    GLfloat vect_2[] = { point_3[0] - point_2[0], point_3[1] - point_2[1],
            point_3[2] - point_2[2] };

    norm[0] = vect_1[1] * vect_2[2] - vect_1[2] * vect_2[1];
    norm[1] = vect_1[2] * vect_2[0] - vect_1[0] * vect_2[2];
    norm[2] = vect_1[0] * vect_2[1] - vect_1[1] * vect_2[0];
}

void draw_square(void)
{
    GLfloat point_1[] = { 0.0, 0.0, 0.0 };
    GLfloat point_2[] = { 10.0, 0.0, 0.0 };
    GLfloat point_3[] = { 10.0, 10.0, 0.0 };
    GLfloat point_4[] = { 0.0, 10.0, 0.0 };

    // compute normal vector
    GLfloat norm[3];
    compute_normal_vector(point_3, point_2, point_1, norm);

    glBegin(GL_QUADS);
        glNormal3fv(norm);
        glVertex3fv(point_1);
        glVertex3fv(point_2);
        glVertex3fv(point_3);
        glVertex3fv(point_4);
    glEnd();
}

void draw_little_square(void)
{
    GLfloat point_1[] = { 0.0, 0.0, 0.0 };
    GLfloat point_2[] = { 1.0, 0.0, 0.0 };
    GLfloat point_3[] = { 1.0, 1.0, 0.0 };
    GLfloat point_4[] = { 0.0, 1.0, 0.0 };

    GLfloat norm[3];

    glPushMatrix();

        glTranslatef(4.5, 4.5, 0.0);

        glBegin(GL_QUADS);
        compute_normal_vector(point_3, point_2, point_1, norm);
        glNormal3fv(norm);
        glVertex3fv(point_1);
        glVertex3fv(point_2);
        glVertex3fv(point_3);
        glVertex3fv(point_4);
        glEnd();

    glPopMatrix();
}

unsigned long get_time(void)
{

#ifdef _WIN32

    SYSTEMTIME* myTime = new SYSTEMTIME;
    GetSystemTime (myTime);

    return
        myTime->wHour * 1000 * 60 * 60 +
        myTime->wMinute * 1000 * 60 +
        myTime->wSecond * 1000 +
        myTime->wMilliseconds;

#else

    timespec tp;
    int ret_value_clock_gettime;
    if ((ret_value_clock_gettime = clock_gettime(CLOCK_REALTIME, &tp)))
    {
        printf("Got error when calling clock_gettime : %d\n",
                ret_value_clock_gettime);
        exit (-1);

    }
    return tp.tv_sec * 1000 + tp.tv_nsec / 1000000;

#endif
}

// find shadow projection matrix
void shadow_matrix_calc(GLfloat shadow_mat[4][4], GLfloat ground_plane[4],
        GLfloat light_pos[4])
{

    // scalar product
    GLfloat dot = ground_plane[0] * light_pos[0] + ground_plane[1]
            * light_pos[1] + ground_plane[2] * light_pos[2] + ground_plane[3]
            * light_pos[3];

    shadow_mat[0][0] = dot - light_pos[0] * ground_plane[0];
    shadow_mat[1][0] = 0.0 - light_pos[0] * ground_plane[1];
    shadow_mat[2][0] = 0.0 - light_pos[0] * ground_plane[2];
    shadow_mat[3][0] = 0.0 - light_pos[0] * ground_plane[3];

    shadow_mat[0][1] = 0.0 - light_pos[1] * ground_plane[0];
    shadow_mat[1][1] = dot - light_pos[1] * ground_plane[1];
    shadow_mat[2][1] = 0.0 - light_pos[1] * ground_plane[2];
    shadow_mat[3][1] = 0.0 - light_pos[1] * ground_plane[3];

    shadow_mat[0][2] = 0.0 - light_pos[2] * ground_plane[0];
    shadow_mat[1][2] = 0.0 - light_pos[2] * ground_plane[1];
    shadow_mat[2][2] = dot - light_pos[2] * ground_plane[2];
    shadow_mat[3][2] = 0.0 - light_pos[2] * ground_plane[3];

    shadow_mat[0][3] = 0.0 - light_pos[3] * ground_plane[0];
    shadow_mat[1][3] = 0.0 - light_pos[3] * ground_plane[1];
    shadow_mat[2][3] = 0.0 - light_pos[3] * ground_plane[2];
    shadow_mat[3][3] = dot - light_pos[3] * ground_plane[3];
}

// find plane from thee points
void find_plane(GLfloat plane[4], GLfloat v0[3], GLfloat v1[3], GLfloat v2[3])
{
    GLfloat vec0[3], vec1[3];

    // need two vectors
    vec0[0] = v1[0] - v0[0];
    vec0[1] = v1[1] - v0[1];
    vec0[2] = v1[2] - v0[2];

    vec1[0] = v2[0] - v0[0];
    vec1[1] = v2[1] - v0[1];
    vec1[2] = v2[2] - v0[2];

    // vectorial product
    plane[0] = vec0[1] * vec1[2] - vec0[2] * vec1[1];
    plane[1] = vec0[2] * vec1[0] - vec0[0] * vec1[2];
    plane[2] = vec0[0] * vec1[1] - vec0[1] * vec1[0];

    plane[3] = -(plane[0] * v0[0] + plane[1] * v0[1] + plane[2] * v0[2]);
}

