// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "engine.h"

// import global variables
extern Board* board;
extern Worm* worm;

extern direction_tp asked_direction;


// navigation engine class
// ----------------------------------------------------------------------------

// default constructor
Engine::Engine(void)
{

    this->cell_marks = new bool*[board->get_x_size()];
    for (int i = 0; i < board->get_x_size(); i++)
        this->cell_marks[i] = new bool[board->get_y_size()];

    activ = true;
}

// reinit the engine
void Engine::init(void)
{
    // reinit the marks
    for (int i = 0; i < board->get_x_size(); i++)
        for (int j = 0; j < board->get_y_size(); j++)
            this->cell_marks[i][j] = false;
}

// activate / decativate auto-pilot
void Engine::enable_disable_engine(void)
{
    activ = !activ;
}


// Get the next direction to be done
direction_tp Engine::get_next(void)
{
    if (!activ)
    {
        return this->asked_direction;
    }

    this->init();

    int head_x;
    int head_y;
    worm->get_head_pos(head_x, head_y);

    Cell* cell = board->get_cell(head_x, head_y);

    CTree<Cell_Node>* tr = new CTree<Cell_Node>();
    tr->item = new Cell_Node(cell);
    tr->parent = NULL;
    this->cell_marks[head_x][head_y] = true;

    // 1. get the list of cells to be followed to reach the seed
    std::list<CTree<Cell_Node>*>* visited_cells = new std::list<CTree<Cell_Node>*>();
    visited_cells->push_front(tr);
    std::list<Cell*>* cell_travel_list = build_cell_travel_list(visited_cells);

    // emtpy visited cell list
    while (visited_cells->size() > 0)
    {
        std::list<CTree<Cell_Node>*>::iterator iterator = visited_cells->begin();
        CTree<Cell_Node>* cell_tree = *iterator;
        delete cell_tree;
        visited_cells->pop_front();
    }
    visited_cells->clear();
    delete visited_cells;

    //printf ("cell_travel_list == 0 ? %d", cell_travel_list == 0);
    //fflush(stdout);

    if (cell_travel_list != NULL && !cell_travel_list->empty())
    {
        std::list<Cell*>::iterator iterator = cell_travel_list->begin();
        iterator++; // we want the second cell in path
        direction_tp ret_direction = cell->get_cell_direction (*iterator);
        delete cell_travel_list;
        return ret_direction;
    }

    // 2. If that list is empty, get the list of directions to be followed to
    //    make the longest possible travel
    // TODO FIXME

    // (In the meantime use get dumb)
    return this->get_dumb();

}

std::list<Cell*>* Engine::build_cell_travel_list(std::list<CTree<Cell_Node>*>* visited_cells)
{
    int seed_x;
    int seed_y;
    board->get_seed_pos(seed_x, seed_y);
    CTree<Cell_Node>* seed_cell_tree = NULL;

    // 0. loop while there are cells to visit in the list
    while (visited_cells->size() > 0)
    {

        std::list<CTree<Cell_Node>*>::iterator iterator = visited_cells->begin();

        // 1. get and remove cell tree in front of list
        CTree<Cell_Node>* cell_tree = *iterator;
        Cell_Node* cell_node = cell_tree->item;
        Cell* cell = cell_node->get_cell();
        visited_cells->pop_front();

        int cell_pos_x;
        int cell_pos_y;
        cell->get_pos(cell_pos_x, cell_pos_y);
        //printf ("handling current cell : %d  - %d\n", cell_pos_x, cell_pos_y);

        // 2. Get each of its neighbour not marked so far
        std::vector<Cell*>* neighbours = cell->get_neighbours();

        for (unsigned int i = 0; i < neighbours->size(); i++)
        {
            Cell* neighbour = neighbours->at (i);
            int n_pos_x;
            int n_pos_y;
            neighbour->get_pos(n_pos_x, n_pos_y);
            if (neighbour->is_free() && !this->cell_marks[n_pos_x][n_pos_y])
            {

                // 3. Build a tree for each these neihgbours
                CTree<Cell_Node>* neighbour_tree = new CTree<Cell_Node>();
                neighbour_tree->item = new Cell_Node(neighbour);
                neighbour_tree->parent = cell_tree;

                // 4. Populate children of the cell with these neighbours
                cell_tree->children.push_back(neighbour_tree);

                // 5. If one of this neighbour is the seed cell then break, we found it... TODO
                if (seed_x == n_pos_x && seed_y == n_pos_y)
                {
                    seed_cell_tree = neighbour_tree;
                    neighbours->clear();
                    delete neighbours;
                    goto done_visit_cells_loop;
                }

                // 6. Mark these neighbours
                this->cell_marks[n_pos_x][n_pos_y] = true;

                // 7. Add each these neighbour at end of the list
                visited_cells->push_back(neighbour_tree);
            }
        }

        // 8. Delete neighbour list
        neighbours->clear();
        delete neighbours;
    }

    done_visit_cells_loop:

    // If seed could not be reached, return null
    if (seed_cell_tree == NULL)
    {
        return NULL;
    }

    // 10. Build returned list
    std::list<Cell*>* cell_travel_list = new std::list<Cell*>();
    CTree<Cell_Node>* current = seed_cell_tree;
    while (current != NULL)
    {
        cell_travel_list->push_front(current->item->get_cell());
        current = current->parent;
    }

    return cell_travel_list;
}

// dummy way to get a direction in case the worm is trapped
direction_tp Engine::get_dumb(void)
{
    static direction_tp dir = down;
    static int dumb_count = 0;
    int head_x, head_y;
    bool do_loop;

    do
    {
        if (dumb_count > 5)
        {
            dumb_count = 0;
            return down; // fuck !!
        }

        do_loop = false;

        if (dir == down)
            dir = right;
        else if (dir == right)
            dir = up;
        else if (dir == up)
            dir = left;
        else if (dir == left)
            dir = down;

        worm->get_head_pos(head_x, head_y);
        switch (dir)
        {
        case right:
            if ((head_x + 1 > board->get_x_size() - 1)
                    || (!board->get_cell(head_x + 1, head_y)->is_free()))
                do_loop = true;
            break;
        case left:
            if ((head_x - 1 < 0)
                    || (!board->get_cell(head_x - 1, head_y)->is_free()))
                do_loop = true;
            break;
        case up:
            if ((head_y + 1 > board->get_y_size() - 1)
                    || (!board->get_cell(head_x, head_y + 1)->is_free()))
                do_loop = true;
            break;
        case down:
            if ((head_y - 1 < 0)
                    || (!board->get_cell(head_x, head_y - 1)->is_free()))
                do_loop = true;
            break;
        }

        dumb_count++;
    }
    while (do_loop);

    dumb_count = 0;
    return dir;
}

void Engine::set_asked_direction (direction_tp asked_direction)
{
    this->asked_direction = asked_direction;
}


// class of a node in a tree structure
// (used by the engine)
// ----------------------------------------------------------------------------

Cell_Node::Cell_Node(Cell* icell)
{
    this->cell = icell;
    visited = false;
}

Cell_Node::~Cell_Node(void)
{
    this->cell = NULL; // DON'T DELETE CELL !!!
}


bool Cell_Node::is_visited()
{
    return this->visited;
}

void Cell_Node::set_visited (bool visited)
{
    this->visited = visited;
}

Cell* Cell_Node::get_cell()
{
    return this->cell;
}



// class of a Tree element in a tree structure
// (used by the engine)
// ----------------------------------------------------------------------------

template<class CItem>
CTree<CItem>::~CTree(void)
{
    delete this->item;
}
