// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#ifndef ENGINE_H
#define ENGINE_H

#include "common.h"
#include "board.h"
#include "worm.h"

// class of a node in a tree structure
// (used by the engine)
// ----------------------------------------------------------------------------
class Cell_Node
{

private:

    Cell* cell;

    bool visited;

public:

    Cell_Node(Cell*);

    // destructor, delete the board cells
    ~Cell_Node(void);

    Cell* get_cell();

    bool is_visited();

    void set_visited(bool);

};

// class of a Tree element in a tree structure
// (used by the engine)
// ----------------------------------------------------------------------------
template<class CItem> class CTree
{

public:

    // destructor, delete the board cells
    ~CTree(void);

    // Tree Item properties
    CItem* item;

    // parent tree
    CTree* parent;

    // Children items
    std::vector<CTree*> children;
};

// navigation engine class
// ----------------------------------------------------------------------------
class Engine
{
private:

    std::list<Cell*>* build_cell_travel_list(std::list<CTree<Cell_Node>*>*);

    direction_tp get_dumb(void);

    bool** cell_marks;

    bool activ; // ON/OFF

    direction_tp asked_direction;

public:

    // default constructor
    Engine(void);

    // reinitialize navigation engine
    void init(void);

    // Used to get the next action
    direction_tp get_next(void);

    // turn auto-pilot on/off
    void enable_disable_engine(void);

    void set_asked_direction (direction_tp asked_direction);
};

#endif
