// ============================================================================
//
// Copyright (c) 2010, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================



#include "worm.h"

extern Board* board;
extern Engine* engine;
extern Worm* worm;

// class of the worm
// ----------------------------------------------------------------------------

// constructor : fix initial position
Worm::Worm(int x, int y, direction_tp dir)
{
    this->body = new Body_Part[board_size * board_size];
    this->init_x = x;
    this->init_y = y;
    this->init_dir = dir;
}

// (re) initialize the worm
void Worm::init(void)
{
    body_part_number = 0;
    offset = 0;
    speed_coeff_worm = 10;

    head.set_pos(init_x, init_y);
    head.set_direction(init_dir);
}

// specifiy the worm speed coefficient
void Worm::fixe_coeff(float coeff)
{
    speed_coeff_worm = coeff;
}

// destructor
Worm::~Worm()
{
    delete[] this->body;
}

// move the worm one unit
void Worm::move(void)
{
    if (offset < offset_range)
        offset += speed_coeff_worm;
}

// how many parts are now in the worm body ?
int Worm::get_body_part_number(void)
{
    return body_part_number;
}

// add a bodypart to the worm
void Worm::add_part(int head_x, int head_y)
{
    int x_pos, y_pos;

    if (body_part_number > 0)
        body[body_part_number - 1].get_pos(x_pos, y_pos);
    else
    {
        x_pos = head_x;
        y_pos = head_y;
    }

    body_part_number++;
    body[body_part_number - 1].set_origin(x_pos, y_pos, body_part_number);

    body[body_part_number - 1].set_direction(
            board->get_cell(x_pos, y_pos)->get_direction());

    board->get_cell(x_pos, y_pos)->set_used();
}

void Worm::handle_head_movement(int x_next_position, int y_next_position, int &quit, int head_x, int head_y)
{
    head.set_pos(x_next_position, y_next_position);
    if (!board->get_cell(x_next_position, y_next_position)->is_free())
        quit = true;
    else
    {
        board->get_cell(head_x, head_y)->set_free();
        board->get_cell(x_next_position, y_next_position)->set_used();
    }
}

// save the new directions
bool Worm::move_cell(void)
{
    int head_x, head_y;
    int new_head_x, new_head_y;
    int x, y;
    int limit;
    bool do_create_seed;

    do_create_seed = false;

    if (offset >= offset_range)
    {
        // ANALYZE HEAD

        // position of the head before the cell switch
        head.get_pos(head_x, head_y);

        // affect to that position the current head direction
        board->get_cell(head_x, head_y)->set_direction(head.get_direction());

        // move the head one complete cell and mark this cell as busy

        // FIXME This can be refactored
        bool quit = false;
        switch (head.get_direction())
        {
        case right:
            if (head_x + 1 > board->get_x_size() - 1)
                quit = true;
            else
            {
                handle_head_movement(head_x + 1, head_y, quit, head_x, head_y);
            }
            break;

        case up:
            if (head_y + 1 > board->get_y_size() - 1)
                quit = true;
            else
            {
                handle_head_movement(head_x, head_y + 1, quit, head_x, head_y);
            }
            break;

        case left:
            if (head_x - 1 < 0)
                quit = true;
            else
            {
                handle_head_movement(head_x - 1, head_y, quit, head_x, head_y);
            }
            break;

        case down:
            if (head_y - 1 < 0)
                quit = true;
            else
            {
                handle_head_movement(head_x, head_y - 1, quit, head_x, head_y);
            }
            break;
        }
        if (quit)
        {
            return false;
        }

        // Then of that new cell has the seed, we reached it ü
        // so we'll have to re-create a seed and add a body part to the worm

        limit = body_part_number;
        // (limt = body_part_number is required to not make the new part move
        // to the next cell along with the worm)

        head.get_pos(new_head_x, new_head_y);
        if (board->is_on_seed_cell(new_head_x, new_head_y))
        {
            this->add_part(head_x, head_y);
            do_create_seed = true;
        }

        // And now the body parts

        // FIXME this can be refactored
        for (int i = 0; i < limit; i++)
        {
            body[i].get_pos(x, y);
            switch (body[i].get_direction())
            {
            case right:
                body[i].set_pos(x + 1, y);
                board->get_cell(x, y)->set_free();
                board->get_cell(x + 1, y)->set_used();
                break;

            case up:
                body[i].set_pos(x, y + 1);
                board->get_cell(x, y)->set_free();
                board->get_cell(x, y + 1)->set_used();
                break;

            case left:
                body[i].set_pos(x - 1, y);
                board->get_cell(x, y)->set_free();
                board->get_cell(x - 1, y)->set_used();
                break;

            case down:
                body[i].set_pos(x, y - 1);
                board->get_cell(x, y)->set_free();
                board->get_cell(x, y - 1)->set_used();
                break;
            }

            // set new firection
            body[i].get_pos(x, y);
            body[i].set_direction(board->get_cell(x, y)->get_direction());
        }

        // reset offset
        offset = 0;

        if (do_create_seed)
        {
            board->create_random_seed();
            do_create_seed = false;
        }

        // get next direction from the engine
        worm->set_direction(engine->get_next());
    }

    return true;
}

// used to know current offset
int Worm::get_offset(void)
{
    return offset;
}

// get the position of the head
void Worm::get_head_pos(int &head_x, int &head_y)
{
    head.get_pos(head_x, head_y);
}

// apply the new direction to the worm (i.e. the head)
void Worm::set_direction(direction_tp dir)
{
    head.set_direction(dir);
}

// usedto know where the camera points to
void Worm::get_pointing_pos(float &x, float &y)
{
    int head_x, head_y;
    head.get_pos(head_x, head_y);

    x = head_x * 10.0;
    y = head_y * 10.0;

    switch (head.get_direction())
    {
    case right:
        x += 10.0 * (float) offset / (float) offset_range;
        break;
    case left:
        x -= 10.0 * (float) offset / (float) offset_range;
        break;
    case up:
        y += 10.0 * (float) offset / (float) offset_range;
        break;
    case down:
        y -= 10.0 * (float) offset / (float) offset_range;
        break;
    }
}

// get the direction of the worm (i.e. the head)
direction_tp Worm::get_direction(void)
{
    return head.get_direction();
}

void Worm::proceed_display(void)
{
    int x, y;

    head.get_pos(x, y);

    glPushMatrix();
        glTranslatef(10.0 * x, 10.0 * y, 0.0);

        switch (head.get_direction())
        {
        case right:
            glTranslatef((float) offset / (float) (offset_range / 10), 0.0, 0.0);
            break;
        case up:
            glTranslatef(0.0, (float) offset / (float) (offset_range / 10), 0.0);
            break;
        case left:
            glTranslatef((float) -offset / (float) (offset_range / 10), 0.0, 0.0);
            break;
        case down:
            glTranslatef(0.0, (float) -offset / (float) (offset_range / 10), 0.0);
            break;
        }

        //glCallList (head.display_list);
        head.proceed_display();

        //head.proceed_display (this->offset);

    glPopMatrix();

    for (int i = 0; i < body_part_number; i++)
    {
        //body[i]->proceed_display (this->offset, this->body_part_number);

        body[i].get_pos(x, y);

        glPushMatrix();

            glTranslatef(10.0 * x, 10.0 * y, 0.0);

            switch (body[i].get_direction()) {
            case right:
                glTranslatef((float) offset / (float) (offset_range / 10), 0.0, 0.0);
                break;
            case up:
                glTranslatef(0.0, (float) offset / (float) (offset_range / 10), 0.0);
                break;
            case left:
                glTranslatef((float) -offset / (float) (offset_range / 10), 0.0,
                        0.0);
                break;
            case down:
                glTranslatef(0.0, (float) -offset / (float) (offset_range / 10),
                        0.0);
                break;
            }

            //glCallList (body[i].display_list);
            body[i].proceed_display();

        glPopMatrix();
    }
}
